@@ -1,5 +1,6 @@
 import os
 import sys
+import inspect
 
 is_py2 = (sys.version_info[0] == 2)
 is_py3 = (sys.version_info[0] == 3)
@@ -20,6 +21,7 @@ if is_py2:
     _str = str
     str = unicode
     range = xrange
+    from itertools import izip
 
     def bytes(b, enc="ascii"):
         return _str(b)
@@ -28,14 +30,15 @@ elif is_py3:
     bytes = bytes
     str = str
     range = range
+    izip = zip
 
 try:
     from urllib.parse import (
-        urlparse, urlunparse, urljoin, quote, unquote, parse_qsl, urlencode
+        urlparse, urlunparse, urljoin, quote, unquote, parse_qsl, urlencode, urlsplit, urlunsplit
     )
     import queue
 except ImportError:
-    from urlparse import urlparse, urlunparse, urljoin, parse_qsl
+    from urlparse import urlparse, urlunparse, urljoin, parse_qsl, urlsplit, urlunsplit
     from urllib import quote, unquote, urlencode
     import Queue as queue
 
@@ -44,7 +47,18 @@ try:
 except ImportError:
     from backports.shutil_which import which
 
+try:
+    from html import unescape as html_unescape
+except ImportError:
+    from HTMLParser import HTMLParser
+    html_unescape = unescape = HTMLParser().unescape
+
+
+
+getargspec = getattr(inspect, "getfullargspec", inspect.getargspec)
+
 
 __all__ = ["is_py2", "is_py3", "is_py33", "is_win32", "str", "bytes",
            "urlparse", "urlunparse", "urljoin", "parse_qsl", "quote",
-           "unquote", "queue", "range", "urlencode", "devnull", "which"]
+           "unquote", "queue", "range", "urlencode", "devnull", "which",
+           "izip", "urlsplit", "urlunsplit", "getargspec", "html_unescape"]
